%Simulations for Fullerton and Heutel, AEJ:EP
%March 2010
%Questions? gaheutel@uncg.edu

clear;
global Y ekl ekz elz ekk ell ezz Ky Ly Z

%%%%%%%%%%%%%%%%%%%%%%%%%
%Define parameters
%%%%%%%%%%%%%%%%%%%%%%%%%

Ky=0.08;
Kx=0.32;
Ly=0.12;
Lx=0.48;  

thetayz = 0.25;

%Given those parameters, the following are defined
X = Kx + Lx;
Y = (Ky + Ly)/(1-thetayz);
K = Kx+Ky;
L = Lx+Ly;
Z = thetayz*Y;

lamdakx=Kx/K;
lamdaky=Ky/K;
lamdalx=Lx/L;
lamdaly=Ly/L;
thetaxl=Lx/X;
thetaxk=Kx/X;
thetayk=Ky/Y;
thetayl=Ly/Y;

gammal = lamdaly/lamdalx;
gammak = lamdaky/lamdakx;

delta = Z/Y;

sigmax = 1;
sigmau = 0.8;

%Elasticities in dirty sector
ekl = .5;
ekz = .5;
elz = .3;

%From these three cross price elasticities, get the own-price elasticities,
%from Allen's identities
ekk = -(1/thetayk)*(thetayl*ekl+thetayz*ekz);
ell = -(1/thetayl)*(thetayk*ekl+thetayz*elz);
ezz = -(1/thetayz)*(thetayk*ekz+thetayl*elz);
%Note that the own-price elasticities cannot be positive. I must check to make sure they are not.
badparams = 0; %An indicator for if the parameters are bad
if ((ekk>0)||(ell>0)||(ezz>0)) 
    disp('Bad Parameters; Positive own-price elasticities');
    badparams=1;
end;

%Now I can use the Allen elasticities to get the production function second
%derivatives, from their definitions
%I call the function solveelas.m
options = optimset();
guess = [-1,1,1,-1.1,1,-1]; %Order of parameters is ykk,ykl,ykz,yll,ylz,yzz
[derivs,mini,flag]=fsolve(@solveelas,guess,options);

ykk = derivs(1); ykl = derivs(2); ykz = derivs(3);
yll = derivs(4); ylz = derivs(5); yzz = derivs(6);
yk = 1;
yl = 1;
yz = 1;
BordHess = [0,yk,yl,yz;yk,ykk,ykl,ykz;yl,ykl,yll,ylz;yz,ykz,ylz,yzz];
F = det(BordHess);
Fzz = cofactor(BordHess,4,4);
Fkk = cofactor(BordHess,2,2);
Fll = cofactor(BordHess,3,3);
Fkl = cofactor(BordHess,2,3);
Fkz = cofactor(BordHess,2,4);
Flz = cofactor(BordHess,3,4);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Performance standard
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Performance standard elasticities
br = -yk*yl/Fzz*(yl/Ky+yk/Ly);
bw = -br;
bdelta = delta*Z*F/Fzz*(ekz-elz);

nu = delta*yz;

%Suppose there is a 10% decrease in the performance standard ratio
deltahat=-0.1;

eta = (thetayk*gammal + thetayl*gammak + gammak*gammal)*(1-nu);
Dps = (1-nu)*sigmau*(thetaxl*thetayk - thetayl*thetaxk)*(gammak-gammal)... 
    + eta*(bw + sigmax*(thetaxl/gammak + thetaxk/gammal));

rhat_ps = (-thetaxl*nu*(gammak-gammal)/Dps + thetaxl*nu*sigmau*(gammak-gammal)/Dps... 
    + thetaxl*eta*bdelta/Dps)*deltahat;
what_ps = (thetaxk*nu*(gammak-gammal)/Dps - thetaxk*nu*sigmau*(gammak-gammal)/Dps... 
    - thetaxk*eta*bdelta/Dps)*deltahat;
pyhat_ps = ((1/Dps)*(thetayk*thetaxl-thetayl*thetaxk)*(-nu*(1-sigmau)*(gammak-gammal)+eta*bdelta)... 
    - nu/(1-nu))*deltahat;
Zhat_ps = (1+thetayk/gammak+thetayl/gammal)^(-1) * (deltahat-pyhat_ps+thetayk*rhat_ps+thetayl*what_ps...
    + (thetayk/gammak+thetayl/gammal)*(deltahat-sigmau*pyhat_ps));

%Decompose r-hat into three effects
sub_eff_ps = thetaxl*eta*bdelta/Dps*deltahat;
out_eff_ps = thetaxl*nu*sigmau*(gammak-gammal)/Dps*deltahat;
outsub_eff_ps = -thetaxl*nu*(gammak-gammal)/Dps*deltahat;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Technology Mandate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

zeta = Z/Ky;
DA2 = F*Z^2/(Y)*(-ekk-ezz+2*ekz); %Denominator in Appendix 2

cr = yl/DA2*(yl/Ky + (yk+zeta*yz)/Ly);
cw = -(yk+zeta*yz)/DA2*(yl/Ky + (yk+zeta*yz)/Ly);
czeta = zeta/DA2*(-yl*yz*(yl/Ky + (yk+zeta*yz)/Ly)+F*Ky*Z/Y*(-ekz+ekk+elz-ekl));

%Suppose there is a 10% decrease in the technology mandate ratio
zetahat=-0.1;

pi = gammak*gammal + thetayk*gammal + thetayl*gammak;
Dtm = sigmau*(thetayk*thetaxl-thetayl*thetaxk)*(gammak-gammal)...
    + sigmax*(thetaxk*gammak+thetaxl*gammal+thetayk+thetayl) - pi*(thetaxl*cr-thetaxk*cw);

rhat_tm = (-thetaxl*nu*(gammak-gammal)/Dtm + thetaxl*nu*sigmau*(gammak-gammal)/Dtm...
    + thetaxl*pi*czeta/Dtm) * zetahat;
what_tm = (thetaxk*nu*(gammak-gammal)/Dtm - thetaxk*nu*sigmau*(gammak-gammal)/Dtm...
    - thetaxk*pi*czeta/Dtm) * zetahat;
pyhat_tm = (((gammak-gammal)*(sigmau-1)*nu + pi*czeta)/Dtm * (thetayk*thetaxl-thetayl*thetaxk) - nu)*zetahat;
Zhat_tm = (-thetaxk*gammak - thetayk - thetaxl*gammal - thetayl)^(-1) * (zetahat*(nu...
    - thetaxk*gammak - thetayk - thetaxl*gammal - thetayl) + sigmau*pyhat_tm...
    - (thetaxl*gammal+thetayl)*(cr*rhat_tm + cw*what_tm + czeta*zetahat));  

%Decompose the effects on r into subsitution, output, output-subsidy,
%and capital-subsidy effects
%For that I need to decompose czeta by defining M and H
M = -zeta/DA2*yl*yz*(yl/Ky + (yk+zeta*yz)/Ly);
H = F*Ky*Z*zeta/(Y*DA2);

sub_eff_tm = thetaxl*pi/Dtm*zetahat*(H*(elz-ekz));
out_eff_tm = thetaxl*nu*sigmau/Dtm*(gammak-gammal)*zetahat;
outsub_eff_tm = -thetaxl*nu/Dtm*(gammak-gammal)*zetahat;
capsub_eff_tm = thetaxl*pi/Dtm*zetahat*(M+H*(ekk-ekl));
M_eff_tm = thetaxl*pi/Dtm*zetahat*M;
